;-----------------------------------------------------------------------------
; Copyright (c) 2006-2007, mike.dld
;-----------------------------------------------------------------------------
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
; FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
; DEALINGS IN THE SOFTWARE.
;-----------------------------------------------------------------------------

format MS COFF

public EXPORTS

include 'proc32.inc'
include 'macros.inc'
;include 'libcommon.inc'
purge section;mov,add,sub

section '.flat' code readable align 16

struct APoint
  X dd ?
  Y dd ?
ends

struct ARect
  Left	 dd ?
  Top	 dd ?
  Right  dd ?
  Bottom dd ?
ends

struct AGfxContext
  NullPoint   APoint
  Caret       APoint
  Pen.Color   dd ?
  Brush.Color dd ?
  Font.Face   dd ?
  Flags       dd ?
ends

GFX_FLAG_DONT_CLOSE = 00000001b

mem.alloc   dd ?
mem.free    dd ?
mem.realloc dd ?
dll.load    dd ?

;-----------------------------------------------------------------------------
proc lib_init ;///////////////////////////////////////////////////////////////
;-----------------------------------------------------------------------------
	mov	[mem.alloc],eax
	mov	[mem.free],ebx
	mov	[mem.realloc],ecx
	mov	[dll.load],edx
	xor	eax,eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.open open? ;/////////////////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	ebx ecx
	xor	ecx,ecx
	cmp	byte[open?],0
	je	.lp1
	mov	eax,12
	mov	ebx,1
	int	0x40
	jmp	@f
  .lp1: or	ecx,GFX_FLAG_DONT_CLOSE
    @@: invoke	mem.alloc,sizeof.AGfxContext
	xor	ebx,ebx
	mov	[eax+AGfxContext.Flags],ecx
	mov	[eax+AGfxContext.NullPoint.X],ebx
	mov	[eax+AGfxContext.NullPoint.Y],ebx
	mov	[eax+AGfxContext.Caret.X],ebx
	mov	[eax+AGfxContext.Caret.Y],ebx
	mov	[eax+AGfxContext.Pen.Color],ebx
	mov	[eax+AGfxContext.Brush.Color],0x00FFFFFF
	pop	ecx ebx
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.close ctx ;//////////////////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax ebx
	mov	ebx,[ctx]
	mov	ebx,[ebx+AGfxContext.Flags]
	invoke	mem.free,[ctx]
	test	ebx,GFX_FLAG_DONT_CLOSE
	jnz	@f
	mov	eax,12
	mov	ebx,2
	int	0x40
    @@: pop	ebx eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.pen.color ctx,color ;////////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax
	mov	eax,[ctx]
	push	[color]
	pop	[eax+AGfxContext.Pen.Color]
	pop	eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.brush.color ctx,color ;//////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax
	mov	eax,[ctx]
	push	[color]
	pop	[eax+AGfxContext.Brush.Color]
	pop	eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.pixel ctx,x,y ;//////////////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax ebx ecx edx
	mov	eax,1
	mov	ebx,[x]
	mov	ecx,[y]
	mov	edx,[ctx]
	mov	edx,[edx+AGfxContext.Pen.Color]
	int	0x40
	pop	edx ecx ebx eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.move.to ctx,x,y ;////////////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax
	mov	eax,[ctx]
	push	[x] [y]
	pop	[eax+AGfxContext.Caret.Y] [eax+AGfxContext.Caret.X]
	pop	eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.line.to ctx,x,y ;////////////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax
	mov	eax,[ctx]
	stdcall gfx.line,eax,[eax+AGfxContext.Caret.X],[eax+AGfxContext.Caret.Y],[x],[y]
	pop	eax
	stdcall gfx.move.to,[ctx],[x],[y]
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.line ctx,x1,y1,x2,y2 ;///////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax ebx ecx edx
	mov	eax,38
	mov	ebx,[x1-2]
	mov	bx,word[x2]
	mov	ecx,[y1-2]
	mov	cx,word[y2]
	mov	edx,[ctx]
	mov	edx,[edx+AGfxContext.Pen.Color]
	int	0x40
	pop	edx ecx ebx eax
	stdcall gfx.move.to,[ctx],[x2],[y2]
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.polyline ctx,points,count ;//////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax ecx
	mov	eax,[points]
	stdcall gfx.move.to,[ctx],[eax+0],[eax+4]
	mov	ecx,[count]
	dec	ecx
	add	eax,8
	stdcall gfx.polyline.to,[ctx],eax,ecx
	pop	ecx eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.polyline.to ctx,points,count ;///////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax ecx
	mov	eax,[points]
	mov	ecx,[count]
    @@: stdcall gfx.line.to,[ctx],[eax+0],[eax+4]
	add	eax,8
	loop	@b
	pop	ecx eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.fillrect ctx,x1,y1,x2,y2 ;///////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax ebx ecx edx
	mov	eax,13
	mov	ebx,[x1-2]
	mov	bx,word[x2]
	sub	bx,word[x1]
	inc	bx
	mov	ecx,[y1-2]
	mov	cx,word[y2]
	sub	cx,word[y1]
	inc	cx
	mov	edx,[ctx]
	mov	edx,[edx+AGfxContext.Brush.Color]
	int	0x40
	pop	edx ecx ebx eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.fillrect.ex ctx,rect ;///////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax
	mov	eax,[rect]
	stdcall gfx.fillrect,[ctx],[eax+ARect.Left],[eax+ARect.Top],[eax+ARect.Right],[eax+ARect.Bottom]
	pop	eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.framerect ctx,x1,y1,x2,y2 ;//////////////////////////////////////////
;-----------------------------------------------------------------------------
	stdcall gfx.move.to,[ctx],[x1],[y1]
	stdcall gfx.line.to,[ctx],[x2],[y1]
	stdcall gfx.line.to,[ctx],[x2],[y2]
	stdcall gfx.line.to,[ctx],[x1],[y2]
	stdcall gfx.line.to,[ctx],[x1],[y1]
	pop	edx ecx ebx eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.framerect.ex ctx,rect ;//////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax
	mov	eax,[rect]
	stdcall gfx.framerect,[ctx],[eax+ARect.Left],[eax+ARect.Top],[eax+ARect.Right],[eax+ARect.Bottom]
	pop	eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.rectangle ctx,x1,y1,x2,y2 ;//////////////////////////////////////////
;-----------------------------------------------------------------------------
	stdcall gfx.framerect,[ctx],[x1],[y1],[x2],[y2]
	push	[y2] [x2] [y1] [x1]
	inc	dword[esp+0x00]
	inc	dword[esp+0x04]
	dec	dword[esp+0x08]
	dec	dword[esp+0x0C]
	stdcall gfx.fillrect,[ctx]
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.rectangle.ex ctx,rect ;//////////////////////////////////////////////
;-----------------------------------------------------------------------------
	push	eax
	mov	eax,[rect]
	stdcall gfx.rectangle,[ctx],[eax+ARect.Left],[eax+ARect.Top],[eax+ARect.Right],[eax+ARect.Bottom]
	pop	eax
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.text ctx,text,len,x,y ;//////////////////////////////////////////////
;-----------------------------------------------------------------------------
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.text.width ctx,text,len ;////////////////////////////////////////////
;-----------------------------------------------------------------------------
	ret
endp

;-----------------------------------------------------------------------------
proc gfx.text.height ctx,text,len ;///////////////////////////////////////////
;-----------------------------------------------------------------------------
	ret
endp

align 16
EXPORTS:

export	\
	lib_init	,'lib_init',\
	0x01000000	,'version',\
	gfx.open	,'gfx.open',\
	gfx.close	,'gfx.close',\
	gfx.pen.color	,'gfx.pen.color',\
	gfx.brush.color ,'gfx.brush.color',\
	gfx.pixel	,'gfx.pixel',\
	gfx.move.to	,'gfx.move.to',\
	gfx.line.to	,'gfx.line.to',\
	gfx.line	,'gfx.line',\
	gfx.polyline	,'gfx.polyline',\
	gfx.polyline.to ,'gfx.polyline.to',\
	gfx.fillrect	,'gfx.fillrect',\
	gfx.fillrect.ex ,'gfx.fillrect.ex',\
	gfx.framerect	,'gfx.framerect',\
	gfx.framerect.ex,'gfx.framerect.ex',\
	gfx.rectangle	,'gfx.rectangle',\
	gfx.rectangle.ex,'gfx.rectangle.ex'
